using AbstractSoftwareInstallationDatabaseImplement.Implements;
using AbstractSoftwareInstallationBusinessLogic;
using AbstractSoftwareInstallationBusinessLogic.BusinessLogic;
using AbstractSoftwareInstallationContracts.BusinessLogicsContracts;
using AbstractSoftwareInstallationContracts.StoragesContracts;
using SoftwareInstallationView;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using NLog.Extensions.Logging;
using AbstractSoftwareInstallationBusinessLogic.OfficePackage;
using AbstractSoftwareInstallationBusinessLogic.OfficePackage.Implements;
using AbstractSoftwareInstallationBusinessLogic.MailWorker;
using AbstractSoftwareInstallationContracts.BindingModels;

namespace SoftwareInstallation
{
    internal static class Program
    {
        private static ServiceProvider? _serviceProvider;
        public static ServiceProvider? ServiceProvider => _serviceProvider;


        /// <summary>
        ///  The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            ApplicationConfiguration.Initialize();
            var services = new ServiceCollection();
            ConfigureServices(services);
            _serviceProvider = services.BuildServiceProvider();
            try
            {
                var mailSender = _serviceProvider.GetService<AbstractMailWorker>();
                mailSender?.MailConfig(new MailConfigBindingModel
                {
                    MailLogin = System.Configuration.ConfigurationManager.AppSettings["MailLogin"] ?? string.Empty,
                    MailPassword = System.Configuration.ConfigurationManager.AppSettings["MailPassword"] ?? string.Empty,
                    SmtpClientHost = System.Configuration.ConfigurationManager.AppSettings["SmtpClientHost"] ?? string.Empty,
                    SmtpClientPort = Convert.ToInt32(System.Configuration.ConfigurationManager.AppSettings["SmtpClientPort"]),
                    PopHost = System.Configuration.ConfigurationManager.AppSettings["PopHost"] ?? string.Empty,
                    PopPort = Convert.ToInt32(System.Configuration.ConfigurationManager.AppSettings["PopPort"])
                });
                //  
                var timer = new System.Threading.Timer(new TimerCallback(MailCheck!), null, 0, 100000);
            }
            catch (Exception ex)
            {
                var logger = _serviceProvider.GetService<ILogger>();
                logger?.LogError(ex, "   ");
            }

            Application.Run(_serviceProvider.GetRequiredService<FormMain>());
        }

        private static void MailCheck(object obj) => ServiceProvider?.GetService<AbstractMailWorker>()?.MailCheck();
        private static void ConfigureServices(ServiceCollection services)
        {
            services.AddLogging(option =>
            {
                option.SetMinimumLevel(LogLevel.Information);
                option.AddNLog("nlog.config");
            });
            services.AddTransient<IMessageInfoStorage, MessageInfoStorage>();
            services.AddTransient<IImplementerStorage, ImplementerStorage>();
            services.AddTransient<IClientStorage, ClientStorage>();
            services.AddTransient<ISoftwareStorage, SoftwareStorage>();
            services.AddTransient<IOrderStorage, OrderStorage>();
            services.AddTransient<IPackageStorage, PackageStorage>();

            services.AddTransient<IMessageInfoLogic, MessageInfoLogic>();
            services.AddTransient<IImplementerLogic, ImplementerLogic>();
            services.AddTransient<IClientLogic, ClientLogic>();
            services.AddTransient<ISoftwareLogic, SoftwareLogic>();
            services.AddTransient<IOrderLogic, OrderLogic>();
            services.AddTransient<IPackageLogic, PackageLogic>();
            services.AddTransient<IReportLogic, ReportLogic>();
            services.AddTransient<IWorkProcess, WorkModeling>();

            services.AddTransient<AbstractSaveToExcel, SaveToExcel>();
            services.AddTransient<AbstractSaveToWord, SaveToWord>();
            services.AddTransient<AbstractSaveToPdf, SaveToPdf>();
            services.AddSingleton<AbstractMailWorker, MailKitWorker>();

            services.AddTransient<FormMails>();
            services.AddTransient<FormImplementers>();
            services.AddTransient<FormImplementer>();
            services.AddTransient<FormMain>();
            services.AddTransient<FormClients>();
            services.AddTransient<FormSoftware>();
            services.AddTransient<FormSoftwares>();
            services.AddTransient<FormCreateOrder>();
            services.AddTransient<FormPackage>();
            services.AddTransient<FormPackageSoftware>();
            services.AddTransient<FormPackages>();
            services.AddTransient<FormReportOrders>();
            services.AddTransient<FormReportPackageSoftwares>();
        }

    }
}
